#include "mig-sockets.h"
int
mig_connect( int fd, char *ip, int port )
{
  	int			flags;
  	int                     select_status;
  	fd_set                  connect_read, connect_write;
	struct timeval          timeout;
  	int getsockopt_length = 0;
  	int getsockopt_error = 	0;
  	struct sockaddr_in      server;
  	timeout.tv_sec = 	CTIMEOUT;
  	timeout.tv_usec = 	0;
  	FD_ZERO( &connect_read );
  	FD_ZERO( &connect_write );
  	FD_SET( fd, &connect_read );
  	FD_SET( fd, &connect_write );
  	bzero( &server, sizeof( server ) );
  	server.sin_family = AF_INET;
  	inet_pton( AF_INET, ip, &server.sin_addr );
  	server.sin_port = htons( port );
  	if( ( flags = fcntl( fd, F_GETFL, 0 ) ) < 0 ) {
    		return( -1 );
  	}
  	if( fcntl( fd, F_SETFL, flags | O_NONBLOCK ) < 0 ) {
    		return( -1 );
  	}
  	if( ( connect( fd, ( struct sockaddr * ) &server, sizeof( server ) ) ) < 0 ) {
    		if( errno != EINPROGRESS ) {
      			return( -1 );
    		}
  	}
  	else {
    		if(fcntl(fd, F_SETFL, flags) < 0) {
      			return (-1);
    		}
    		return( 1 );
  	}
  	select_status = select( fd + 1, &connect_read, &connect_write, NULL, &timeout );
  	if( select_status == 0 ) {
    		return( -1 );
  	}
  	if( select_status == -1 ) {
    		return( -1 );
  	}
  	if( FD_ISSET( fd, &connect_read ) || FD_ISSET( fd, &connect_write ) ) {
    		if( FD_ISSET( fd, &connect_read ) && FD_ISSET( fd, &connect_write ) ) {
      			getsockopt_length = sizeof( getsockopt_error );
      			if( getsockopt( fd, SOL_SOCKET, SO_ERROR, &getsockopt_error, &getsockopt_length ) < 0 ) {
				errno = ETIMEDOUT;
				return( -1 );
      			}
      			if( getsockopt_error == 0 ) {
				if( fcntl( fd, F_SETFL, flags ) < 0 ) {
	  				return (-1);
				}
				return( 1 );
      			}
      			else {
				errno = getsockopt_error;
				return( -1 );
      			}
    		}
  	}
  	else {
    	return( -1 );
  	}
  	if( fcntl( fd, F_SETFL, flags ) < 0 ) {
    		return( -1 );
  	}
  	return( 1 );
}

int
mig_read_timer( int fd )
{
  	int			flags;
  	int			select_status;
  	fd_set			fdread;
  	struct timeval		timeout;
        timeout.tv_sec = 	RTIMEOUT;
        timeout.tv_usec = 	0;
        FD_ZERO( &fdread );
        FD_SET( fd, &fdread );
  	if( ( flags = fcntl( fd, F_GETFL, 0 ) ) < 0 ) {
    		return( -1 );
  	}
  	if( fcntl( fd, F_SETFL, flags | O_NONBLOCK ) < 0 ) {
    		return( -1 );
  	}
  	select_status = select( fd + 1, &fdread, NULL, NULL, &timeout );
  	if( select_status == 0 ) {
    		return( -1 );
  	}
  	if( select_status == -1 ) {
    		return( -1 );
  	}
  	if( FD_ISSET( fd, &fdread ) ) {
 		if( fcntl( fd, F_SETFL, flags ) < 0 ) { 
			return ( -1 );
    		}
    		return( 1 );
  	}
  	else {
    		errno = ETIMEDOUT;
    		return( -1 );
  	}
}

int
mig_write_timer( int fd )
{
  	int			flags;
  	int			select_status;
  	fd_set			fdwrite;
  	struct timeval		timeout;
        timeout.tv_sec =	WTIMEOUT;
        timeout.tv_usec	= 	0;
        FD_ZERO(&fdwrite);
        FD_SET(fd, &fdwrite);
  	if( ( flags = fcntl( fd, F_GETFL, 0 ) ) < 0 ) {
    		return( -1 );
  	}
  	if( fcntl( fd, F_SETFL, flags | O_NONBLOCK ) < 0 ) {
    		return( -1 );
  	}
  	select_status = select( fd + 1, NULL, &fdwrite, NULL, &timeout );
  	if( select_status == 0 ) {
    		return( -1 );
  	}
  	if( select_status == -1 ) {
    		return( -1 );
  	}
  	if( FD_ISSET( fd, &fdwrite ) ) {
    		if( fcntl( fd, F_SETFL, flags ) < 0 ) {
      			return( -1 );
    		}
    		return( 1 );
  	}
  	else {
    		errno = ETIMEDOUT;
    		return( -1 );
  	}
}

// get all readable output, time out when nothing else to read
// return number of bytes read on success
// return -1 on error
int
mig_get_readable( char *buff, int size, int fd ) {
	int count = 		0;
	char 			tmp[ 2 ];
  	bzero( tmp, 2 );
  	bzero( buff, size );
  	while( 1 ) {
    		if( mig_read_timer( fd ) == 1 ) {
      			if( read( fd, tmp, 1 ) <= 0 ) {
        			return( -1 );
      			}
      			else {
				tmp[ 1 ] = '\0';
        			strcat( buff, tmp );
				count++;
      			}
    		}
    		else break;
  	}
  	return( count );
}

// get lines from fd that match a condition
// return number of bytes read on success
// return -1 on error
int
mig_get_matched( char *buff, int size, char *match, int match_chars, int fd ) {
	int count = 		0;
	char 			tmp[ 2 ];
	char 			big[ BUFSIZ ];
	bzero( tmp, 2 );
	bzero( big, BUFSIZ );
	bzero( buff, size );
	while( 1 ) {
		if( mig_read_timer( fd ) == 1 ) {
			if( read( fd, tmp, 1 ) <= 0 ) {
				return( -1 );
			}
			else {
				tmp[ 1 ] = '\0';
				strcat( big, tmp );
				count++;
			}
			if( ( tmp[ 0 ] == '\n' ) || ( tmp[ 0 ] == '\0' ) ) {
				if( !strncmp( big, match, match_chars ) ) {
					strcat( buff, big );
					bzero( big, BUFSIZ );
				}
				else return( count );
			}
		}
		else break;
	}
	return( count );
}

// get one line from fd
// return number of bytes read on success
// return -1 on error
int
mig_get_line( char *buff, int size, int fd ) {
	int count =		0;
        char 			tmp[ 2 ];
        bzero( tmp, 2 );
        bzero( buff, size );
        while( 1 ) {
                if( mig_read_timer( fd ) == 1 ) {
                        if( read( fd, tmp, 1 ) <= 0 ) {
                                return( -1 );
                        }
                        else {
				tmp[ 1 ] = '\0';
                                strcat( buff, tmp );
				count++;
                        }
			if( tmp[ 0 ] == '\n' || tmp[ 0 ] == '\0' ) {
				return( count );
			}
                }
                else break;
        }
        return( count );
}
